@@ -2,19 +2,21 @@ __all__ = [
     'main',
 ]
 
+
 import os
 import sys
 import traceback
 import zipfile
 
-from extract_msg import __doc__, utils
-from extract_msg.enums import AttachErrorBehavior
+from extract_msg import __doc__, openMsg, utils
+from extract_msg.enums import ErrorBehavior
+from typing import List
 
 
-def main() -> None:
+def main(argv: List[str] = sys.argv) -> None:
     # Setup logging to stdout, indicate running from cli
     CLI_LOGGING = 'extract_msg_cli'
-    args = utils.getCommandArgs(sys.argv[1:])
+    args = utils.getCommandArgs(argv[1:])
 
     # Determine where to save the files to.
     currentDir = os.getcwd() # Store this in case the path changes.
@@ -49,6 +51,7 @@ def main() -> None:
         'extractEmbedded': args.extractEmbedded,
         'html': args.html,
         'json': args.json,
+        'overwriteExisting': args.overwriteExisting,
         'pdf': args.pdf,
         'preparedHtml': args.preparedHtml,
         'rtf': args.rtf,
@@ -64,13 +67,13 @@ def main() -> None:
     }
 
     openKwargs = {
-        'ignoreRtfDeErrors': args.ignoreRtfDeErrors,
+        'errorBehavior': ErrorBehavior.RTFDE if args.ignoreRtfDeErrors else ErrorBehavior.THROW,
     }
 
     # If we are skipping the NotImplementedError attachments, we need to
     # suppress the error.
     if args.skipNotImplemented:
-        openKwargs['attachmentErrorBehavior'] = AttachErrorBehavior.NOT_IMPLEMENTED
+        openKwargs['errorBehavior'] |= ErrorBehavior.ATTACH_NOT_IMPLEMENTED
 
     def strSanitize(inp):
         """
@@ -91,7 +94,7 @@ def main() -> None:
             except UnicodeEncodeError:
                 print(f'Saving file "{strSanitize(x)}" (failed to print without repr)...')
         try:
-            with utils.openMsg(x, **openKwargs) as msg:
+            with openMsg(x, **openKwargs) as msg:
                 if args.dumpStdout:
                     print(msg.body)
                 elif args.noFolders:
@@ -109,4 +112,4 @@ def main() -> None:
         _zip.close()
 
 if __name__ == '__main__':
-    main()
+    main(sys.argv)
