@@ -10,10 +10,15 @@ __all__ = [
 import copy
 import re
 
-from typing import Dict, Iterator, List, Optional, Tuple, TYPE_CHECKING
+from typing import (
+        Dict, Iterator, List, Optional, SupportsBytes, Tuple, TYPE_CHECKING,
+        Union
+    )
 
 from . import constants
+from .constants import MSG_PATH
 from .enums import Color, DirectoryEntryType
+from .exceptions import TooManySectorsError
 from .utils import ceilDiv, dictGetCasedKey, inputToMsgPath
 from olefile.olefile import OleDirectoryEntry, OleFileIO
 from red_black_dict_mod import RedBlackTree
@@ -21,42 +26,42 @@ from red_black_dict_mod import RedBlackTree
 
 # Allow for nice type checking.
 if TYPE_CHECKING:
-    from .msg import MSGFile
-
+    from .msg_classes import MSGFile
 
 
 class DirectoryEntry:
     """
     An internal representation of a stream or storage in the OleWriter.
+
     Originals should be inaccessible outside of the class.
     """
-    name : str = ''
-    rightChild : 'DirectoryEntry' = None
-    leftChild : 'DirectoryEntry' = None
-    childTreeRoot : 'DirectoryEntry' = None
-    stateBits : int = 0
-    creationTime : int = 0
-    modifiedTime : int = 0
-    type : DirectoryEntryType = DirectoryEntryType.UNALLOCATED
+    name: str = ''
+    rightChild: Optional[DirectoryEntry] = None
+    leftChild: Optional[DirectoryEntry] = None
+    childTreeRoot: Optional[DirectoryEntry] = None
+    stateBits: int = 0
+    creationTime: int = 0
+    modifiedTime: int = 0
+    type: DirectoryEntryType = DirectoryEntryType.UNALLOCATED
 
     # These get set after things have been sorted by the red black tree.
-    id : int = -1
+    id: int = -1
     # This is the ID for the left child. The terminology in the docs is really
     # annoying.
-    leftSiblingID : int = 0xFFFFFFFF
-    rightSiblingID : int = 0xFFFFFFFF
+    leftSiblingID: int = 0xFFFFFFFF
+    rightSiblingID: int = 0xFFFFFFFF
     # This is the ID for the root of the child tree, if any.
-    childID : int = 0xFFFFFFFF
-    startingSectorLocation : int = 0
-    color : Color = Color.BLACK
+    childID: int = 0xFFFFFFFF
+    startingSectorLocation: int = 0
+    color: Color = Color.BLACK
 
-    clsid : bytes = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
-    data : bytes = b''
+    clsid: bytes = b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+    data: bytes = b''
 
-    def __init__(self):
-        pass
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
 
-    def toBytes(self):
+    def toBytes(self) -> bytes:
         """
         Converts the entry to bytes to be writen to a file.
         """
@@ -70,7 +75,7 @@ class DirectoryEntry:
 
         nameBytes = self.name.encode('utf-16-le')
 
-        return constants.ST_CF_DIR_ENTRY.pack(
+        return constants.st.ST_CF_DIR_ENTRY.pack(
                                               nameBytes,
                                               len(nameBytes) + 2,
                                               self.type,
@@ -93,7 +98,7 @@ class OleWriter:
     Takes data to write to a compound binary format file, as specified in
     [MS-CFB].
     """
-    def __init__(self, rootClsid : bytes = constants.DEFAULT_CLSID):
+    def __init__(self, rootClsid: bytes = constants.DEFAULT_CLSID):
         self.__rootEntry = DirectoryEntry()
         self.__rootEntry.name = "Root Entry"
         self.__rootEntry.type = DirectoryEntryType.ROOT_STORAGE
@@ -101,27 +106,30 @@ class OleWriter:
         # The root entry will always exist, so this must be at least 1.
         self.__dirEntryCount = 1
         self.__dirEntries = {}
-        self.__largeEntries = []
+        self.__largeEntries: List[DirectoryEntry] = []
         self.__largeEntrySectors = 0
         self.__numMinifatSectors = 0
 
-    def __getContainingStorage(self, path : List[str], entryExists : bool = True, create : bool = False) -> Dict:
+        # In a future version, this will be setable as an optional argument.
+        self.__version = 3
+
+    def __getContainingStorage(self, path: List[str], entryExists: bool = True, create: bool = False) -> Dict:
         """
-        Finds the storage dict internally where the entry specified by
-        :param path: would be created. If :param create: is True, missing
-        storages will be created with default settings.
+        Finds the storage ``dict`` internally where the entry specified by
+        :param path: would be created.
 
-        :param entryExists: If True, throws an error when the requested entry
-            does not yet exist.
-        :param create: If True, creates missing storages with default settings.
+        :param entryExists: If ``True``, throws an error when the requested
+            entry does not yet exist.
+        :param create: If ``True``, creates missing storages with default
+            settings.
 
-        :raises OSError: If :param create: is False and the path could not be
-            found. Also raised if :param entryExists: is True and the requested
-            entry does not exist.
+        :raises OSError: If :param create: is ``False`` and the path could not
+            be found. Also raised if :param entryExists: is ``True`` and the
+            requested entry does not exist.
         :raises ValueError: Tried to access an interal stream or tried to use
-            both the create option and the entryExists option as True.
+            both the create option and the entryExists option is ``True``.
 
-        :returns: The storage dict that the entry is in.
+        :returns: The storage ``dict`` that the entry is in.
         """
         if not path:
             raise OSError('Path cannot be empty.')
@@ -158,9 +166,9 @@ class OleWriter:
 
         return _dir
 
-    def __getEntry(self, path : List[str]) -> DirectoryEntry:
+    def __getEntry(self, path: List[str]) -> DirectoryEntry:
         """
-        Finds and returns an existing DirectoryEntry instance in the writer.
+        Finds and returns an existing ``DirectoryEntry`` instance in the writer.
 
         :raises OSError: If the entry does not exist.
         :raises ValueError: If access to an internal item is attempted.
@@ -172,10 +180,11 @@ class OleWriter:
         else:
             return item
 
-    def __modifyEntry(self, entry : DirectoryEntry, **kwargs):
+    def __modifyEntry(self, entry: DirectoryEntry, **kwargs):
         """
-        Edits the DirectoryEntry with the data provided. Common code used for
-        :method addEntry: and :method editEntry:.
+        Edits the DirectoryEntry with the data provided.
+
+        Common code used for :meth:`addEntry` and :meth:`editEntry`.
 
         :raises TypeError: Attempted to modify the data of a storage.
         :raises ValueError: Some part of the data given to modify the various
@@ -195,7 +204,15 @@ class OleWriter:
             if entry.type is not DirectoryEntryType.STREAM:
                 raise TypeError('Cannot set the data of a storage object.')
             if not isinstance(data, bytes):
-                raise ValueError('Data must be a bytes instance if set.')
+                try:
+                    data = bytes(data)
+                except Exception:
+                    raise ValueError('Data must be a bytes instance or convertable to bytes if set.')
+            # Check the length of data. In future versions, this may be a
+            # different check which is done when swapping between version 3 and
+            # 4 of the compound file binary file format.
+            if len(data) > 0x80000000:
+                raise ValueError('Current version of extract_msg does not support streams greater than 2 GB in OLE files.')
 
         if clsid is not None:
             if not isinstance(clsid, bytes):
@@ -219,7 +236,6 @@ class OleWriter:
             if not isinstance(stateBits, int) or stateBits < 0 or stateBits > 0xFFFFFFFF:
                 raise ValueError('State bits must be a positive 4 byte int.')
 
-
         # Now that all our checks have passed, let's set our data.
         if data is not None:
             entry.data = data
@@ -242,7 +258,6 @@ class OleWriter:
         self.__largeEntries.clear()
         self.__largeEntrySectors = 0
 
-        count = 0
         for entry in self.__walkEntries():
             self.__dirEntryCount += 1
             if entry.type == DirectoryEntryType.STREAM:
@@ -250,12 +265,13 @@ class OleWriter:
                     self.__numMinifatSectors += ceilDiv(len(entry.data), 64)
                 else:
                     self.__largeEntries.append(entry)
-                    self.__largeEntrySectors += ceilDiv(len(entry.data), 512)
+                    self.__largeEntrySectors += ceilDiv(len(entry.data), self.__sectorSize)
 
     def __walkEntries(self) -> Iterator[DirectoryEntry]:
         """
-        Returns a generator that will walk the entires recursively. Each item
-        returned by it will be a DirectoryEntry instance.
+        Returns a generator that will walk the entires recursively.
+
+        Each item returned by it will be a DirectoryEntry instance.
         """
         toProcess = [self.__dirEntries]
         yield self.__rootEntry
@@ -269,12 +285,35 @@ class OleWriter:
                     else:
                         yield item
 
+    @property
+    def __dirEntsPerSector(self) -> int:
+        """
+        The number of Directory Entries that can fit in a sector.
+        """
+        return self.__sectorSize // 128
+
+    @property
+    def __linksPerSector(self) -> int:
+        """
+        The number of links per FAT/DIFAT sector.
+        """
+        return self.__sectorSize // 4
+
+    @property
+    def __miniSectorsPerSector(self) -> int:
+        """
+        The number of mini sectors that a regular sector will hold.
+        """
+        return self.__sectorSize // 64
+
     @property
     def __numberOfSectors(self) -> int:
-        """
-        TODO: finish the calculation needed. For now this just notes how many
-        sectors are needed for the directory entries.
-        """
+        # Most of this should be pretty self evident, but line by line the
+        # calculation is as such:
+        # 1. How many sectors are needed for the directory entries.
+        # 2. How many FAT sectors are needed for the MiniStream.
+        # 3. How many sectors are needed for the MiniFat (ceil divide #2 by 16).
+        # 4. The number of FAT sectors needed to store the larger data.
         return ceilDiv(self.__dirEntryCount, 4) + \
                self.__numMinifat + \
                ceilDiv(self.__numMinifat, 16) + \
@@ -283,9 +322,16 @@ class OleWriter:
     @property
     def __numMinifat(self) -> int:
         """
-        The number of FAT sectors needed to store the mini FAT.
+        The number of FAT sectors needed to store the mini stream.
         """
-        return ceilDiv(self.__numMinifatSectors, 8)
+        return ceilDiv(64 * self.__numMinifatSectors, self.__sectorSize)
+
+    @property
+    def __sectorSize(self) -> int:
+        """
+        The size of each sector, in bytes.
+        """
+        return 512 if self.__version == 3 else 4096
 
     def _cleanupEntries(self) -> None:
         """
@@ -309,17 +355,17 @@ class OleWriter:
         """
         # Right now we just use an annoying while loop to get the numbers.
         numDifat = 0
-        # All divisions are ceiling divisions, so we leave them as
-        numFat = ceilDiv(self.__numberOfSectors, 127) or 1
+        # All divisions are ceiling divisions,.
+        numFat = ceilDiv(self.__numberOfSectors or 1, self.__linksPerSector - 1)
         newNumFat = 1
         while numFat != newNumFat:
             numFat = newNumFat
-            numDifat = ceilDiv(max(numFat - 109, 0), 127)
-            newNumFat = ceilDiv(self.__numberOfSectors + numDifat, 127)
+            numDifat = ceilDiv(max(numFat - 109, 0), self.__linksPerSector - 1)
+            newNumFat = ceilDiv(self.__numberOfSectors + numDifat, self.__linksPerSector - 1)
 
         return (numFat, numDifat, self.__numberOfSectors + numDifat + numFat)
 
-    def _treeSort(self, startingSector : int) -> List[DirectoryEntry]:
+    def _treeSort(self, startingSector: int) -> List[DirectoryEntry]:
         """
         Uses red-black trees to sort the internal data in preparation for
         writing the file, returning a list, in order, of the entries to write.
@@ -328,7 +374,7 @@ class OleWriter:
         root = copy.copy(self.__rootEntry)
 
         # Add the location of the start of the mini stream.
-        root.startingSectorLocation = (startingSector + ceilDiv(self.__dirEntryCount, 4) + ceilDiv(self.__numMinifatSectors, 128)) if self.__numMinifat > 0 else 0xFFFFFFFE
+        root.startingSectorLocation = (startingSector + ceilDiv(self.__dirEntryCount, 4) + ceilDiv(self.__numMinifatSectors, self.__linksPerSector)) if self.__numMinifat > 0 else 0xFFFFFFFE
         root.streamSize = self.__numMinifatSectors * 64
         root.childTreeRoot = None
         root.childID = 0xFFFFFFFF
@@ -409,52 +455,67 @@ class OleWriter:
 
     def _writeBeginning(self, f) -> int:
         """
-        Writes the beginning to the file :param f:. This includes the header,
-        DIFAT, and FAT blocks.
+        Writes the beginning to the file :param f:.
+
+        This includes the header, DIFAT, and FAT blocks.
 
         :returns: The current sector number after all the data is written.
+
+        :raises TooMuchDataError: The number of sectors required for the file is
+            too large.
         """
         # Recalculate some things needed for saving.
         self.__recalculateSectors()
         # Since we are going to need these multiple times, get them now.
         numFat, numDifat, totalSectors = self._getFatSectors()
 
+        # Check to make sure there isn't too much data to write.
+        if totalSectors > 0xFFFFFFFB:
+            raise TooManySectorsError('Data in OleWriter requires too many sectors to write to a version 3 file.')
+
+        # The ministream *cannot* be greater than 2 GB, so check that before
+        # writing anything. A minifat sector is 64 bytes, so the maximum amount
+        # of them is 0x2000000.
+        if self.__numMinifatSectors > 0x2000000:
+            raise TooManySectorsError('Data is OleWriter requires too many MiniFAT sectors.')
+
         # Header signature.
         f.write(b'\xD0\xCF\x11\xE0\xA1\xB1\x1A\xE1')
         # Header CLSID.
         f.write(b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')
         # Minor version.
         f.write(b'\x3E\x00')
-        # Major version. For now, we only support version 3.
-        f.write(b'\x03\x00')
+        # Major version. For now, we only support version 3, but support for
+        # version 4 is planned.
+        f.write(b'\x03\x00' if self.__version == 3 else b'\x04\x00')
         # Byte order. Specifies that it is little endian.
         f.write(b'\xFE\xFF')
         # Sector shift.
-        f.write(b'\x09\x00')
+        f.write(b'\x09\x00' if self.__version == 3 else b'\x0C\x00')
         # Mini sector shift.
         f.write(b'\x06\x00')
         # Reserved.
         f.write(b'\x00\x00\x00\x00\x00\x00')
         # Number of directory sectors. Version 3 says this *must* be 0.
-        f.write(constants.ST_LE_UI32.pack(0))
+        f.write(constants.st.ST_LE_UI32.pack(0))
         # Number of FAT sectors.
-        f.write(constants.ST_LE_UI32.pack(numFat))
+        f.write(constants.st.ST_LE_UI32.pack(numFat))
         # First directory sector location (Sector for the directory stream).
         # We place that right after the DIFAT and FAT.
-        f.write(constants.ST_LE_UI32.pack(numFat + numDifat))
+        f.write(constants.st.ST_LE_UI32.pack(numFat + numDifat))
         # Transation signature number.
         f.write(b'\x00\x00\x00\x00')
         # Mini stream cutoff size.
         f.write(b'\x00\x10\x00\x00')
         # First mini FAT sector location.
-        f.write(constants.ST_LE_UI32.pack((numFat + numDifat + ceilDiv(self.__dirEntryCount, 4)) if self.__numMinifat > 0 else 0xFFFFFFFE))
+        f.write(constants.st.ST_LE_UI32.pack((numFat + numDifat + ceilDiv(self.__dirEntryCount, 4)) if self.__numMinifat > 0 else 0xFFFFFFFE))
         # Number of mini FAT sectors.
-        f.write(constants.ST_LE_UI32.pack(ceilDiv(self.__numMinifatSectors, 128)))
+        f.write(constants.st.ST_LE_UI32.pack(ceilDiv(self.__numMinifatSectors, self.__linksPerSector)))
         # First DIFAT sector location. If there are none, set to 0xFFFFFFFE (End
         # of chain).
-        f.write(constants.ST_LE_UI32.pack(0 if numDifat else 0xFFFFFFFE))
+        f.write(constants.st.ST_LE_UI32.pack(0 if numDifat else 0xFFFFFFFE))
         # Number of DIFAT sectors.
-        f.write(constants.ST_LE_UI32.pack(numDifat))
+        f.write(constants.st.ST_LE_UI32.pack(numDifat))
 
         # To make life easier on me, I'm having the code start with the DIFAT
         # followed by the FAT sectors, as I can write them all at once before
@@ -462,16 +523,20 @@ class OleWriter:
 
         # Write the DIFAT sectors.
         for x in range(numFat):
+            # Quickly check if we have hit 109. If we have, and we are writing
+            # a version 4 file, we need to pad a bunch of null bytes.
+            if x == 109 and self.__version == 4:
+                f.write(b'\x00' * 3584)
             # This kind of sucks to code, ngl.
-            if x > 109 and (x - 109) % 127 == 0:
+            if x > 109 and (x - 109) % (self.__linksPerSector - 1) == 0:
                 # If we are at the end of a DIFAT sector, write the jump.
-                f.write(constants.ST_LE_UI32.pack((x - 109) // 127))
+                f.write(constants.st.ST_LE_UI32.pack((x - 109) // (self.__linksPerSector - 1)))
             # Write the next FAT sector location.
-            f.write(constants.ST_LE_UI32.pack(x + numDifat))
+            f.write(constants.st.ST_LE_UI32.pack(x + numDifat))
 
         # Finally, fill out the last DIFAT sector with null entries.
         if numFat > 109:
-            f.write(b'\xFF\xFF\xFF\xFF' * (127 - ((numFat - 109) % 127)))
+            f.write(b'\xFF\xFF\xFF\xFF' * ((self.__linksPerSector - 1) - ((numFat - 109) % (self.__linksPerSector - 1))))
             # Finally, make sure to write the end of chain marker for the DIFAT.
             f.write(b'\xFE\xFF\xFF\xFF')
         else:
@@ -488,19 +553,19 @@ class OleWriter:
         offset = numDifat + numFat
 
         # Fill in the values for the directory stream.
-        for x in range(offset + 1, offset + ceilDiv(self.__dirEntryCount, 4)):
-            f.write(constants.ST_LE_UI32.pack(x))
+        for x in range(offset + 1, offset + ceilDiv(self.__dirEntryCount, self.__dirEntsPerSector)):
+            f.write(constants.st.ST_LE_UI32.pack(x))
 
         # Write the end of chain marker.
         f.write(b'\xFE\xFF\xFF\xFF')
 
-        offset += ceilDiv(self.__dirEntryCount, 4)
+        offset += ceilDiv(self.__dirEntryCount, self.__dirEntsPerSector)
 
         # Check if we have minifat *at all* first.
         if self.__numMinifatSectors > 0:
             # Mini FAT chain.
             for x in range(offset + 1, offset + ceilDiv(self.__numMinifat, 16)):
-                f.write(constants.ST_LE_UI32.pack(x))
+                f.write(constants.st.ST_LE_UI32.pack(x))
 
             # Write the end of chain marker.
             f.write(b'\xFE\xFF\xFF\xFF')
@@ -509,7 +574,7 @@ class OleWriter:
 
             # The mini stream sectors.
             for x in range(offset + 1, offset + self.__numMinifat):
-                f.write(constants.ST_LE_UI32.pack(x))
+                f.write(constants.st.ST_LE_UI32.pack(x))
 
             # Write the end of chain marker.
             f.write(b'\xFE\xFF\xFF\xFF')
@@ -521,10 +586,10 @@ class OleWriter:
         # to that list if the size was more than 4096. The order in the list is
         # how they will eventually be stored into the file correctly.
         for entry in self.__largeEntries:
-            size = ceilDiv(len(entry.data), 512)
+            size = ceilDiv(len(entry.data), self.__sectorSize)
             entry.startingSectorLocation = offset
             for x in range(offset + 1, offset + size):
-                f.write(constants.ST_LE_UI32.pack(x))
+                f.write(constants.st.ST_LE_UI32.pack(x))
 
             # Write the end of chain marker.
             f.write(b'\xFE\xFF\xFF\xFF')
@@ -532,14 +597,14 @@ class OleWriter:
             offset += size
 
         # Finally, fill fat with markers to specify no block exists.
-        freeSectors = totalSectors & 0x7F
+        freeSectors = totalSectors & (self.__linksPerSector - 1)
         if freeSectors:
-            f.write(b'\xFF\xFF\xFF\xFF' * (128 - freeSectors))
+            f.write(b'\xFF\xFF\xFF\xFF' * (self.__linksPerSector - freeSectors))
 
         # Finally, return the current sector index for use in other places.
         return numDifat + numFat
 
-    def _writeDirectoryEntries(self, f, startingSector : int) -> List[DirectoryEntry]:
+    def _writeDirectoryEntries(self, f, startingSector: int) -> List[DirectoryEntry]:
         """
         Writes out all the directory entries. Returns the list generated.
         """
@@ -551,11 +616,11 @@ class OleWriter:
 
         return entries
 
-    def _writeDirectoryEntry(self, f, entry : DirectoryEntry) -> None:
+    def _writeDirectoryEntry(self, f, entry: DirectoryEntry) -> None:
         """
         Writes the directory entry to the file f.
         """
-        f.write(entry.toBytes())
+        f.write(bytes(entry))
 
     def _writeFinal(self, f) -> None:
         """
@@ -564,10 +629,10 @@ class OleWriter:
         """
         for x in self.__largeEntries:
             f.write(x.data)
-            if len(x.data) & 511:
-                f.write(b'\x00' * (512 - (len(x.data) & 511)))
+            if len(x.data) & (self.__sectorSize - 1):
+                f.write(b'\x00' * (self.__sectorSize - (len(x.data) & (self.__sectorSize - 1))))
 
-    def _writeMini(self, f, entries : List[DirectoryEntry]) -> None:
+    def _writeMini(self, f, entries: List[DirectoryEntry]) -> None:
         """
         Writes the mini FAT followed by the full mini stream.
         """
@@ -577,14 +642,14 @@ class OleWriter:
             if x.type == DirectoryEntryType.STREAM and len(x.data) < 4096:
                 size = ceilDiv(len(x.data), 64)
                 for x in range(currentSector + 1, currentSector + size):
-                    f.write(constants.ST_LE_UI32.pack(x))
+                    f.write(constants.st.ST_LE_UI32.pack(x))
                 if size > 0:
                     f.write(b'\xFE\xFF\xFF\xFF')
                 currentSector += size
 
         # Finally, write the remaining slots.
-        if currentSector & 127:
-            f.write(b'\xFF\xFF\xFF\xFF' * (128 - (currentSector & 127)))
+        if currentSector & (self.__linksPerSector - 1):
+            f.write(b'\xFF\xFF\xFF\xFF' * (self.__linksPerSector - (currentSector & (self.__linksPerSector - 1))))
 
         # Write the mini stream.
         for x in entries:
@@ -594,10 +659,10 @@ class OleWriter:
                     f.write(b'\x00' * (64 - (len(x.data) & 63)))
 
         # Pad the final mini stream block.
-        if self.__numMinifatSectors & 7:
-            f.write((b'\x00' * 64) * (8 - (self.__numMinifatSectors & 7)))
+        if self.__numMinifatSectors & (self.__miniSectorsPerSector - 1):
+            f.write((b'\x00' * 64) * (self.__miniSectorsPerSector - (self.__numMinifatSectors & (self.__miniSectorsPerSector - 1))))
 
-    def addEntry(self, path, data : bytes = None, storage : bool = False, **kwargs) -> None:
+    def addEntry(self, path: MSG_PATH, data: Optional[Union[bytes, SupportsBytes]] = None, storage: bool = False, **kwargs) -> None:
         """
         Adds an entry to the OleWriter instance at the path specified, adding
         storages with default settings where necessary. If the entry is not a
@@ -605,9 +670,10 @@ class OleWriter:
 
         :param path: The path to add the entry at. Must not contain a path part
             that is an already added stream.
-        :param data: The bytes for a stream.
-        :param storage: If True, the entry to add is a storage. Otherwise, the
-            entry is a stream.
+        :param data: The bytes for a stream or an object with a ``__bytes__``
+            method.
+        :param storage: If ``True``, the entry to add is a storage. Otherwise,
+            the entry is a stream.
         :param clsid: The CLSID for the stream/storage. Must a a bytes instance
             that is 16 bytes long.
         :param creationTime: An 8 byte filetime int. Sets the creation time of
@@ -617,8 +683,10 @@ class OleWriter:
         :param stateBits: A 4 byte int. Sets the state bits, user-defined flags,
             of the entry. For a stream, this *SHOULD* be unset.
 
-        :raises OSError: A stream was found on the path before the end or an entry with the same name already exists.
+        :raises OSError: A stream was found on the path before the end or an
+            entry with the same name already exists.
         :raises ValueError: Attempts to access an internal item.
+        :raises ValueError: The data provided is too large.
         """
         path = inputToMsgPath(path)
         # First, find the current place in our dict to add the item.
@@ -638,13 +706,14 @@ class OleWriter:
         else:
             _dir[path[-1]] = entry
 
-    def addOleEntry(self, path, entry : OleDirectoryEntry, data : Optional[bytes] = None) -> None:
+    def addOleEntry(self, path: MSG_PATH, entry: OleDirectoryEntry, data: Optional[Union[bytes, SupportsBytes]] = None) -> None:
         """
         Uses the entry provided to add the data to the writer.
 
         :raises OSError: Tried to add an entry to a path that has not yet
             been added, tried to add as a child of a stream, or tried to add an
             entry where one already exists under the same name.
+        :raises ValueError: The data provided is too large.
         """
         path = inputToMsgPath(path)
         # First, find the current place in our dict to add the item.
@@ -658,27 +727,32 @@ class OleWriter:
         newEntry = DirectoryEntry()
         if entry.entry_type == DirectoryEntryType.STORAGE:
             # Handle a storage entry.
-            # First add the dict to our tree of items.
-            _dir[path[-1]] = {'::DirectoryEntry': newEntry}
-
-            # Finally, setup the values for the stream.
+            # First, setup the values for the storage.
             newEntry.name = entry.name
             newEntry.type = DirectoryEntryType.STORAGE
             newEntry.clsid = _unClsid(entry.clsid)
             newEntry.stateBits = entry.dwUserFlags
             newEntry.creationTime = entry.createTime
             newEntry.modifiedTime = entry.modifyTime
+
+            # Finally add the dict to our tree of items.
+            _dir[path[-1]] = {'::DirectoryEntry': newEntry}
         else:
             # Handle a stream entry.
-            # First add the entry to out dict of entries.
-            _dir[path[-1]] = newEntry
+            # First, setup the values for the stream.
             newEntry.name = entry.name
             newEntry.type = DirectoryEntryType.STREAM
             newEntry.clsid = _unClsid(entry.clsid)
             newEntry.stateBits = entry.dwUserFlags
 
-            # Finally, handle the data.
-            newEntry.data = data or b''
+            # Next, handle the data.
+            data = data or b''
+            newEntry.data = bytes(data)
+            if len(newEntry.data) > 0x80000000:
+                raise ValueError('Current version of extract_msg does not support streams greater than 2 GB in OLE files.')
+
+            # Finally add the entry to out dict of entries.
+            _dir[path[-1]] = newEntry
 
         self.__dirEntryCount += 1
 
@@ -700,13 +774,13 @@ class OleWriter:
         # path does remember the case used.
         del _dir[dictGetCasedKey(_dir, path[-1])]
 
-    def editEntry(self, path, **kwargs) -> None:
+    def editEntry(self, path: MSG_PATH, **kwargs) -> None:
         """
         Used to edit values of an entry by setting the specific kwargs. Set a
         value to something other than None to set it.
 
         :param data: The data of a stream. Will error if used for something
-            other than a stream.
+            other than a stream. Must be bytes or convertable to bytes.
         :param clsid: The CLSID for the stream/storage. Must a a bytes instance
             that is 16 bytes long.
         :param creationTime: An 8 byte filetime int. Sets the creation time of
@@ -716,7 +790,6 @@ class OleWriter:
         :param stateBits: A 4 byte int. Sets the state bits, user-defined flags,
             of the entry. For a stream, this *SHOULD* be unset.
 
-
         To convert a 32 character hexadecial CLSID into the bytes for this
         function, the _unClsid function in the ole_writer submodule can be used.
 
@@ -731,7 +804,7 @@ class OleWriter:
         # Send it to be modified using the arguments given.
         self.__modifyEntry(entry, **kwargs)
 
-    def fromMsg(self, msg : MSGFile) -> None:
+    def fromMsg(self, msg: MSGFile) -> None:
         """
         Copies the streams and stream information necessary from the MSG file.
         """
@@ -745,7 +818,7 @@ class OleWriter:
 
         for x in entries:
             entry = msg._getOleEntry(x)
-            data = msg._getStream(x) if entry.entry_type == DirectoryEntryType.STREAM else None
+            data = msg.getStream(x) if entry.entry_type == DirectoryEntryType.STREAM else None
             # THe properties stream on embedded messages actualy needs to be
             # transformed a little (*why* it is like that is a mystery to me).
             # Basically we just need to add a "reserved" section to it in a
@@ -768,9 +841,9 @@ class OleWriter:
             # Create our generator.
             gen = (x for x in msg._oleListDir() if len(x) > 1 and x[0] == '__nameid_version1.0')
             for x in gen:
-                self.addOleEntry(x, msg._getOleEntry(x, prefix = False), msg._getStream(x, prefix = False))
+                self.addOleEntry(x, msg._getOleEntry(x, prefix = False), msg.getStream(x, prefix = False))
 
-    def fromOleFile(self, ole : OleFileIO, rootPath = []) -> None:
+    def fromOleFile(self, ole: OleFileIO, rootPath: MSG_PATH = []) -> None:
         """
         Copies all the streams from the proided OLE file into this writer.
 
@@ -780,9 +853,9 @@ class OleWriter:
         modification of an embedded properties stream when extracting an
         embedded MSG file.
 
-        :param rootPath: A path (accepted by olefile.OleFileIO) to the directory
-            to use as the root of the file. If not provided, the file root will
-            be used.
+        :param rootPath: A path (accepted by ``olefile.OleFileIO``) to the
+            directory to use as the root of the file. If not provided, the file
+            root will be used.
 
         :raises OSError: If :param rootPath: does not exist in the file.
         """
@@ -827,9 +900,9 @@ class OleWriter:
 
             self.addOleEntry(x, entry, data)
 
-    def getEntry(self, path) -> DirectoryEntry:
+    def getEntry(self, path: MSG_PATH) -> DirectoryEntry:
         """
-        Finds and returns a copy of an existing DirectoryEntry instance in the
+        Finds and returns a copy of an existing `DirectoryEntry` instance in the
         writer. Use this method to check the internal status of an entry.
 
         :raises OSError: If the entry does not exist.
@@ -837,12 +910,13 @@ class OleWriter:
         """
         return copy.copy(self.__getEntry(inputToMsgPath(path)))
 
-    def listItems(self, streams = True, storages = False) -> List[List[str]]:
+    def listItems(self, streams: bool = True, storages: bool = False) -> List[List[str]]:
         """
         Returns a list of the specified items currently in the writter.
 
-        :param streams: If True, includes the path for each stream in the list.
-        :param storages: If True, includes the path for each storage in the
+        :param streams: If ``True``, includes the path for each stream in the
+            list.
+        :param storages: If ``True``, includes the path for each storage in the
             list.
         """
         # We are actually abusing the walk function a bit here to life much
@@ -867,7 +941,7 @@ class OleWriter:
         paths.sort()
         return paths
 
-    def renameEntry(self, path, newName : str) -> None:
+    def renameEntry(self, path: MSG_PATH, newName: str) -> None:
         """
         Changes the name of an entry, leaving it in it's current position.
 
@@ -879,7 +953,7 @@ class OleWriter:
         # First, validate the new name.
         if not isinstance(newName, str):
             raise ValueError('New name must be a string.')
-        if constants.RE_INVALID_OLE_PATH.search(newName):
+        if constants.re.INVALID_OLE_PATH.search(newName):
             raise ValueError('Invalid character(s) in new name. Must not contain the following characters: \\//!:')
         if len(newName) > 31:
             raise ValueError('New name must be less than 32 characters.')
@@ -916,9 +990,9 @@ class OleWriter:
 
     def walk(self) -> Iterator[Tuple[List[str], List[str], List[str]]]:
         """
-        Functional equivelent to :function os.walk:, but for going over the file
-        structure of the OLE file to be written. Unlike :function os.walk:, it
-        takes no arguments.
+        Functional equivelent to ``os.walk``, but for going over the file
+        structure of the OLE file to be written. Unlike ``os.walk``, it takes
+        no arguments.
 
         :returns: A tuple of three lists. The first is the path, as a list of
             strings, for the directory (or an empty list for the root), the
@@ -946,8 +1020,15 @@ class OleWriter:
 
     def write(self, path) -> None:
         """
-        Writes the data to the path specified. If :param path: has a write
-        method it will use the object directly.
+        Writes the data to the path specified.
+
+        If :param path: has a ``write`` method, the object will be used
+        directly.
+
+        If a failure occurs, the file or IO device may have been modified.
+
+        :raises TooManySectorsError: The number of sectors requires for a part
+            of writing is too large.
         """
         opened = False
 
@@ -975,9 +1056,9 @@ class OleWriter:
 
 
 
-def _unClsid(clsid : str) -> bytes:
+def _unClsid(clsid: str) -> bytes:
     """
-    Converts the clsid from olefile.olefile._clsid back to bytes.
+    Converts the clsid from ``olefile.olefile._clsid`` back to bytes.
     """
     if not clsid:
         return b''
