@@ -10,10 +10,12 @@ __all__ = [
 import io
 import struct
 
-from typing import Any, Tuple, Union
+from typing import Any, Optional, Tuple, Type, TypeVar, Union
 
 from .. import constants
 
+_T = TypeVar('_T')
+
 
 class BytesReader(io.BytesIO):
     """
@@ -21,33 +23,33 @@ class BytesReader(io.BytesIO):
     stream.
     """
 
-    def __init__(self, *args, littleEndian = True, **kwargs):
+    def __init__(self, *args, littleEndian: bool = True, **kwargs):
         super().__init__(*args, **kwargs)
         self.__le = bool(littleEndian)
         if self.__le:
-            self.__int8_t = constants.ST_LE_I8
-            self.__int16_t = constants.ST_LE_I16
-            self.__int32_t = constants.ST_LE_I32
-            self.__int64_t = constants.ST_LE_I64
-            self.__uint8_t = constants.ST_LE_UI8
-            self.__uint16_t = constants.ST_LE_UI16
-            self.__uint32_t = constants.ST_LE_UI32
-            self.__uint64_t = constants.ST_LE_UI64
-            self.__float_t = constants.ST_LE_F32
-            self.__double_t = constants.ST_LE_F64
+            self.__int8_t = constants.st.ST_LE_I8
+            self.__int16_t = constants.st.ST_LE_I16
+            self.__int32_t = constants.st.ST_LE_I32
+            self.__int64_t = constants.st.ST_LE_I64
+            self.__uint8_t = constants.st.ST_LE_UI8
+            self.__uint16_t = constants.st.ST_LE_UI16
+            self.__uint32_t = constants.st.ST_LE_UI32
+            self.__uint64_t = constants.st.ST_LE_UI64
+            self.__float_t = constants.st.ST_LE_F32
+            self.__double_t = constants.st.ST_LE_F64
         else:
-            self.__int8_t = constants.ST_BE_I8
-            self.__int16_t = constants.ST_BE_I16
-            self.__int32_t = constants.ST_BE_I32
-            self.__int64_t = constants.ST_BE_I64
-            self.__uint8_t = constants.ST_BE_UI8
-            self.__uint16_t = constants.ST_BE_UI16
-            self.__uint32_t = constants.ST_BE_UI32
-            self.__uint64_t = constants.ST_BE_UI64
-            self.__float_t = constants.ST_BE_F32
-            self.__double_t = constants.ST_BE_F64
+            self.__int8_t = constants.st.ST_BE_I8
+            self.__int16_t = constants.st.ST_BE_I16
+            self.__int32_t = constants.st.ST_BE_I32
+            self.__int64_t = constants.st.ST_BE_I64
+            self.__uint8_t = constants.st.ST_BE_UI8
+            self.__uint16_t = constants.st.ST_BE_UI16
+            self.__uint32_t = constants.st.ST_BE_UI32
+            self.__uint64_t = constants.st.ST_BE_UI64
+            self.__float_t = constants.st.ST_BE_F32
+            self.__double_t = constants.st.ST_BE_F64
 
-    def _readDecodedString(self, encoding, width : int = 1) -> str:
+    def _readDecodedString(self, encoding: str, width: int = 1) -> str:
         """
         Reads a null terminated string with the specified character width
         decoded using the specified encoding. If it cannot be read or cannot be
@@ -61,7 +63,7 @@ class BytesReader(io.BytesIO):
                 self.seek(position)
             raise
 
-    def assertNull(self, length : int, errorMsg : str = None) -> bytes:
+    def assertNull(self, length: int, errorMsg: Optional[str] = None) -> bytes:
         """
         Reads the number of bytes specified and ensures they are all null.
 
@@ -91,7 +93,7 @@ class BytesReader(io.BytesIO):
 
         return valueRead
 
-    def assertRead(self, value : bytes, errorMsg : str = None) -> bytes:
+    def assertRead(self, value: bytes, errorMsg: Optional[str] = None) -> bytes:
         """
         Reads the number of bytes and compares them to the value provided. If it
         does not match, throws a value error.
@@ -114,7 +116,7 @@ class BytesReader(io.BytesIO):
         if len(value) == 0:
             return b''
 
-        if not isinstance(value, bytes):
+        if not isinstance(value, bytes): # pyright: ignore
             raise TypeError(':param value: was not bytes.')
 
         valueRead = self.tryReadBytes(len(value))
@@ -149,7 +151,7 @@ class BytesReader(io.BytesIO):
         else:
             raise IOError('Not enough bytes left in buffer.')
 
-    def readByteString(self, width : int = 1) -> bytes:
+    def readByteString(self, width: int = 1) -> bytes:
         """
         Reads a string of bytes until it finds the null character, returning
         everything before that and consuming the null. Unlike other string
@@ -163,7 +165,6 @@ class BytesReader(io.BytesIO):
 
         position = self.tell()
         string = b''
-        endFound = False;
         null = b'\x00' * width
 
         while True:
@@ -181,7 +182,7 @@ class BytesReader(io.BytesIO):
                 # Otherwise add the character to our string.
                 string += nextChar
 
-    def readClass(self, _class):
+    def readClass(self, _class: Type[_T]) -> _T:
         """
         Takes anything with a __SIZE__ property and a call function that takes
         a single bytes argument and returns the result of that function.
@@ -190,6 +191,8 @@ class BytesReader(io.BytesIO):
         instance of the class created with that amount of bytes. However, there
         is little reason to truly limit it to only that.
         """
+        if not hasattr(_class, '__SIZE__'):
+            raise TypeError('Argument to readClass MUST have a __SIZE__ attribute.')
         value = self.tryReadBytes(_class.__SIZE__)
         if value:
             return _class(value)
@@ -246,7 +249,7 @@ class BytesReader(io.BytesIO):
         else:
             raise IOError('Not enough bytes left in buffer.')
 
-    def readStruct(self, _struct : Union[struct.Struct, Any]) -> Tuple:
+    def readStruct(self, _struct: Union[struct.Struct, Any]) -> Tuple[Any, ...]:
         """
         Read enough bytes for a struct and unpack it, returning the tuple of
         values.
@@ -323,7 +326,7 @@ class BytesReader(io.BytesIO):
         """
         return self._readDecodedString('utf-32-le' if self.__le else 'utf-32-be', 4)
 
-    def tryReadBytes(self, size : int) -> bytes:
+    def tryReadBytes(self, size: int) -> bytes:
         """
         Tries to read the specified number of bytes, returning b'' if not
         possible. Will only change the position of the read pointer if reading
