@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+
 __all__ = [
     'AddressBookEntryID',
     'ContactAddressEntryID',
@@ -13,12 +16,19 @@ __all__ = [
 ]
 
 
+import abc
 import logging
-from typing import Union
+
+from typing import Optional, Union
 
 from ._helpers import BytesReader
 from .. import constants
-from ..enums import AddressBookType, ContactAddressIndex, DisplayType, EntryIDType, MacintoshEncoding, MessageFormat, MessageType, OORBodyFormat
+from ..enums import (
+        AddressBookType, ContactAddressIndex, DisplayType, EntryIDType,
+        MacintoshEncoding, MessageFormat, MessageType, OORBodyFormat,
+        WrappedType
+    )
+from ..exceptions import FeatureNotImplemented
 from ..utils import bitwiseAdjustedAnd, bytesToGuid
 
 
@@ -27,7 +37,7 @@ logger.addHandler(logging.NullHandler())
 
 
 # First we define the main EntryID structure that is the base for the others.
-class EntryID:
+class EntryID(abc.ABC):
     """
     Base class for all EntryID structures. Use :classmethod autoCreate: to
     automatically create the correct EntryID structure type from the specified
@@ -35,12 +45,15 @@ class EntryID:
     """
 
     @classmethod
-    def autoCreate(cls, data) -> 'EntryID':
+    def autoCreate(cls, data: Optional[bytes]) -> Optional[EntryID]:
         """
         Automatically determines the type of EntryID and returns an instance of
         the correct subclass. If the subclass cannot be determined, will return
         a plain EntryID instance.
         """
+        if not data:
+            return None
+
         if len(data) < 20:
             raise ValueError('Cannot create an EntryID with less than 20 bytes.')
         providerUID = data[4:20]
@@ -78,15 +91,19 @@ class EntryID:
         if providerUID == EntryIDType.WRAPPED:
             return WrappedEntryID(data)
 
-        logger.warn(f'UID for EntryID found in database, but no class was specified for it: {providerUID}')
-        # If all else fails and we do recognize it, just return a plain EntryID.
-        return cls(data)
+        raise FeatureNotImplemented(f'UID for EntryID found in database, but no class was specified for it: {providerUID}')
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         self.__flags = data[:4]
         self.__providerUID = data[4:20]
         self.__rawData = data
 
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
+
     @property
     def flags(self) -> bytes:
         """
@@ -112,6 +129,14 @@ class EntryID:
         """
         return self.__flags == b'\x00\x00\x00\x00'
 
+    @property
+    @abc.abstractmethod
+    def position(self) -> int:
+        """
+        Used to tell the amount of bytes read in this EntryID. Useful for
+        EntryID data that has been chained together with no separator.
+        """
+
     @property
     def providerUID(self) -> bytes:
         """
@@ -119,13 +144,6 @@ class EntryID:
         """
         return self.__providerUID
 
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw bytes used in this Entry ID.
-        """
-        return self.__rawData
-
 
 
 # Now for the specific types.
@@ -134,7 +152,7 @@ class AddressBookEntryID(EntryID):
     An Address Book EntryID structure, as specified in [MS-OXCDATA].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
         # Version *MUST* be 1.
@@ -144,6 +162,11 @@ class AddressBookEntryID(EntryID):
 
         self.__type = AddressBookType(reader.readUnsignedInt())
         self.__X500DN = reader.readByteString()
+        self.__position = reader.tell() + 20
+
+    @property
+    def position(self) -> int:
+        return self.__position
 
     @property
     def type(self) -> AddressBookType:
@@ -175,19 +198,20 @@ class ContactAddressEntryID(EntryID):
     object.
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
-        if reader.readUnsignedInt() != 3:
-            raise ValueError(f'Version must be 3 (got {self.__version}).')
-        if reader.readUnsignedInt() != 5:
-            raise ValueError(f'Type must be 4 (got {self.__version}).')
+        if (version := reader.readUnsignedInt()) != 3:
+            raise ValueError(f'Version must be 3 (got {version}).')
+        if (type_ := reader.readUnsignedInt()) != 4:
+            raise ValueError(f'Type must be 4 (got {type_}).')
         self.__index = ContactAddressIndex(reader.readUnsignedInt())
         self.__entryIdCount = reader.readUnsignedInt()
         self.__entryID = MessageEntryID(reader.read(self.__entryIdCount))
+        self.__position = reader.tell() + 20
 
     @property
-    def entryID(self) -> 'MessageEntryID':
+    def entryID(self) -> MessageEntryID:
         """
         The EntryID contained in this object.
         """
@@ -207,6 +231,10 @@ class ContactAddressEntryID(EntryID):
         """
         return self.__index
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
 
 
 class FolderEntryID(EntryID):
@@ -214,15 +242,15 @@ class FolderEntryID(EntryID):
     A Folder EntryID structure, as defined in [MS-OXCDATA].
     """
 
-    __SIZE__ : int = 46
+    __SIZE__: int = 46
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
         self.__folderType = MessageType(reader.readUnsignedShort())
         self.__databaseGuid = bytesToGuid(reader.read(16))
         # This entry is 6 bytes, so we pull some shenanigans to unpack it.
-        self.__globalCounter = constants.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')
+        self.__globalCounter = constants.st.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')[0]
         reader.assertNull(2, 'Pad bytes were not 0.')
 
     @property
@@ -247,6 +275,10 @@ class FolderEntryID(EntryID):
         """
         return self.__globalCounter
 
+    @property
+    def position(self) -> int:
+        return self.__SIZE__
+
 
 
 class MessageEntryID(EntryID):
@@ -254,22 +286,22 @@ class MessageEntryID(EntryID):
     A Message EntryID structure, as defined in [MS-OXCDATA].
     """
 
-    __SIZE__ : int = 70
+    __SIZE__: int = 70
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
         self.__messageType = MessageType(reader.readUnsignedShort())
         self.__folderDatabaseGuid = bytesToGuid(reader.read(16))
         # This entry is 6 bytes, so we pull some shenanigans to unpack it.
-        self.__folderGlobalCounter = constants.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')
+        self.__folderGlobalCounter = constants.st.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')[0]
         reader.assertNull(2, 'Pad bytes were not 0.')
         self.__messageDatabaseGuid = bytesToGuid(reader.read(16))
         # This entry is 6 bytes, so we pull some shenanigans to unpack it.
-        self.__messageGlobalCounter = constants.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')
+        self.__messageGlobalCounter = constants.st.ST_LE_UI64.unpack(reader.read(6) + b'\x00\x00')[0]
         reader.assertNull(2, 'Pad bytes were not 0.')
         # Not sure why Microsoft decided to say "yes, let's do 2 6-byte integers
-        # followed by 2 pad bits each" instead of just 2 8-byte integers with a
+        # followed by 2 pad bytes each" instead of just 2 8-byte integers with a
         # maximum value, but here we are.
 
     @property
@@ -310,6 +342,10 @@ class MessageEntryID(EntryID):
         """
         return self.__messageType
 
+    @property
+    def position(self) -> int:
+        return self.__SIZE__
+
 
 
 class NNTPNewsgroupFolderEntryID(EntryID):
@@ -317,28 +353,33 @@ class NNTPNewsgroupFolderEntryID(EntryID):
     A NNTP Newsgroup Folder EntryID structure, as defined in [MS-OXCDATA].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
         self.__folderType = reader.readUnsignedShort()
         if self.__folderType != 0x000C:
             raise ValueError(f'Folder type was not 0x000C (got {self.__folderType})')
-        self.__newsgroupName = reader.readAnsiString()
+        self.__newsgroupName = reader.readByteString()
+        self.__position = reader.tell() + 20
 
     @property
     def folderType(self) -> int:
         """
-        The type of folder. MUST be 0x000C.
+        The type of folder. MUST be ``0x000C``.
         """
         return self.__folderType
 
     @property
-    def newsgroupName(self) -> str:
+    def newsgroupName(self) -> bytes:
         """
-        The name of the newsgroup.
+        The name of the newsgroup, as an ANSI string.
         """
         return self.__newsgroupName
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
 
 
 class OneOffRecipient(EntryID):
@@ -346,7 +387,7 @@ class OneOffRecipient(EntryID):
     A One-Off EntryID structure, as specified in [MS-OXCDATA].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         # Create a reader to easily
         reader = BytesReader(data[20:])
@@ -388,6 +429,8 @@ class OneOffRecipient(EntryID):
             self.__addressType = reader.readByteString()
             self.__emailAddress = reader.readByteString()
 
+        self.__position = reader.tell() + 20
+
     @property
     def addressType(self) -> Union[str, bytes]:
         """
@@ -444,6 +487,10 @@ class OneOffRecipient(EntryID):
         """
         return self.__messageFormat
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
 
 
 class PermanentEntryID(EntryID):
@@ -451,13 +498,15 @@ class PermanentEntryID(EntryID):
     A Permanent EntryID structure, as defined in [MS-OXNSPI].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
-        unpacked = constants.STPEID.unpack(data[:28])
+        reader = BytesReader(data)
+        unpacked = reader.readStruct(constants.st.ST_PEID)
         if unpacked[0] != 0:
             raise TypeError(f'Not a PermanentEntryID (expected 0, got {unpacked[0]}).')
         self.__displayTypeString = DisplayType(unpacked[2])
-        self.__distinguishedName = data[28:-1].decode('ascii') # Cut off the null character at the end and decode the data as ascii
+        self.__distinguishedName = reader.readAsciiString()
+        self.__position = reader.tell()
 
     @property
     def displayTypeString(self) -> DisplayType:
@@ -473,6 +522,10 @@ class PermanentEntryID(EntryID):
         """
         return self.__distinguishedName
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
 
 
 class PersonalDistributionListEntryID(EntryID):
@@ -480,17 +533,18 @@ class PersonalDistributionListEntryID(EntryID):
     A Personal Distribution List EntryID structure, as defined in [MS-OXCDATA].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
-        if reader.readUnsignedInt() != 3:
-            raise ValueError(f'Version must be 3 (got {self.__version}).')
-        if reader.readUnsignedInt() != 5:
-            raise ValueError(f'Type must be 5 (got {self.__version}).')
-        if reader.readUnsignedInt() != 0xFF:
-            raise ValueError(f'Index must be 255 (got {self.__version}).')
+        if (arg := reader.readUnsignedInt()) != 3:
+            raise ValueError(f'Version must be 3 (got {arg}).')
+        if (arg := reader.readUnsignedInt()) != 5:
+            raise ValueError(f'Type must be 5 (got {arg}).')
+        if (arg := reader.readUnsignedInt()) != 0xFF:
+            raise ValueError(f'Index must be 255 (got {arg}).')
         self.__entryIdCount = reader.readUnsignedInt()
         self.__entryID = MessageEntryID(reader.read(self.__entryIdCount))
+        self.__position = reader.tell() + 20
 
     @property
     def entryID(self) -> MessageEntryID:
@@ -506,13 +560,18 @@ class PersonalDistributionListEntryID(EntryID):
         """
         return self.__entryIdCount
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
+
 
 class StoreObjectEntryID(EntryID):
     """
     A Store Object EntryID structure, as defined in [MS-OXCDATA].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         reader = BytesReader(data[20:])
 
@@ -530,6 +589,18 @@ class StoreObjectEntryID(EntryID):
         if self.__wrappedFlags != 0:
             raise ValueError(f'Wrapped flags was not set to 0 (got {self.__wrappedFlags}).')
 
+        self.__wrappedProviderUID = reader.read(16)
+        self.__wrappedType = WrappedType(reader.readUnsignedInt())
+        # Don't know how this is encoded, just that it is "single-byte
+        # characters".
+        self.__serverShortname = reader.readByteString()
+        if self.__wrappedProviderUID == b'\x1B\x55\xFA\x20\xAA\x66\x11\xCD\x9B\xC8\x00\xAA\x00\x2F\xC4\x5A':
+            self.__mailboxDN = reader.readAsciiString()
+        else:
+            self.__mailboxDN = None
+
+        self.__position = reader.tell() + 20
+
     @property
     def dllFileName(self) -> bytes:
         """
@@ -541,17 +612,49 @@ class StoreObjectEntryID(EntryID):
     def flag(self) -> int:
         return self.__flag
 
+    @property
+    def mailboxDN(self) -> Optional[str]:
+        """
+        A string representing the X500 DN of the mailbox, as specified in
+        [MS-OXOAB]. THis field is present only for mailbox databases.
+        """
+        return self.__mailboxDN
+
+    @property
+    def position(self) -> int:
+        return self.__position
+
+    @property
+    def serverShortname(self) -> bytes:
+        """
+        A string of single-byte characters indicating the short name or NetBIOS
+        name of the server.
+        """
+        return self.__serverShortname
+
     @property
     def version(self) -> int:
         return self.__version
 
+    @property
+    def wrappedProviderUID(self) -> bytes:
+        return self.__wrappedProviderUID
+
+    @property
+    def wrappedType(self) -> WrappedType:
+        """
+        Determined by where the folder is located.
+        """
+        return self.__wrappedType
+
+
 
 class WrappedEntryID(EntryID):
     """
     A WrappedEntryId structure, as specified in [MS-OXOCNTC].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         super().__init__(data)
         # Grab the type byte and parse it.
         self.__type = data[20]
@@ -566,6 +669,7 @@ class WrappedEntryID(EntryID):
             raise ValueError(f'Found wrapped entry id with invalid type (type bits were {bits}).')
 
         self.__embeddedIsOneOff = self.__type & 0x80 == 0
+        self.__position = 21 + self.__embeddedEntryID.position
 
     @property
     def embeddedEntryID(self) -> EntryID:
@@ -581,6 +685,10 @@ class WrappedEntryID(EntryID):
         """
         return self.__embeddedIsOneOff
 
+    @property
+    def position(self) -> int:
+        return self.__position
+
     @property
     def type(self) -> int:
         """
