@@ -24,13 +24,19 @@ class FolderID:
     A Folder ID structure specified in [MS-OXCDATA].
     """
 
-    __SIZE__ : int = 16
+    __SIZE__: int = 16
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         self.__rawData = data
-        self.__replicaID = constants.STUI16.unpack(data[:2])
+        self.__replicaID = constants.st.ST_LE_UI16.unpack(data[:2])[0]
         # This entry is 6 bytes, so we pull some shenanigans to unpack it.
-        self.__globalCounter = constants.STUI64.unpack(data[2:8] + b'\x00\x00')
+        self.__globalCounter = constants.st.ST_LE_UI64.unpack(data[2:8] + b'\x00\x00')[0]
+
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
 
     @property
     def globalCounter(self) -> int:
@@ -39,13 +45,6 @@ class FolderID:
         """
         return self.__globalCounter
 
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw bytes used to create this object.
-        """
-        return self.__rawData
-
     @property
     def replicaID(self) -> int:
         """
@@ -60,7 +59,7 @@ class GlobalObjectID:
     A GlobalObjectID structure, as specified in [MS-OXOCAL].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         self.__rawData = data
         reader = BytesReader(data)
         expectedBytes = b'\x04\x00\x00\x00\x82\x00\xE0\x00\x74\xC5\xB7\x10\x1A\x82\xE0\x08'
@@ -68,7 +67,7 @@ class GlobalObjectID:
         self.__byteArrayID = reader.assertRead(expectedBytes, errorMsg)
         self.__yh = reader.read(1)
         self.__yl = reader.read(1)
-        self.__year = constants.ST_BE_UI16.unpack(self.__yh + self.__yl)[0]
+        self.__year = constants.st.ST_BE_UI16.unpack(self.__yh + self.__yl)[0]
         self.__month = reader.readUnsignedByte()
         self.__day = reader.readUnsignedByte()
         self.__creationTime = filetimeToDatetime(reader.readUnsignedLong())
@@ -76,6 +75,12 @@ class GlobalObjectID:
         size = reader.readUnsignedInt()
         self.__data = reader.read(size)
 
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
+
     @property
     def byteArrayID(self) -> bytes:
         """
@@ -115,13 +120,6 @@ class GlobalObjectID:
         """
         return self.__month
 
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw bytes used to create this object.
-        """
-        return self.__rawData
-
     @property
     def year(self) -> int:
         """
@@ -137,13 +135,19 @@ class MessageID:
     A Message ID structure, as defined in [MS-OXCDATA].
     """
 
-    __SIZE__ : int = 8
+    __SIZE__: int = 8
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         self.__rawData = data
-        self.__replicaID = constants.STUI16.unpack(data[:2])
+        self.__replicaID = constants.st.ST_LE_UI16.unpack(data[:2])[0]
         # This entry is 6 bytes, so we pull some shenanigans to unpack it.
-        self.__globalCounter = constants.STUI64.unpack(data[2:8] + b'\x00\x00')
+        self.__globalCounter = constants.st.ST_LE_UI64.unpack(data[2:8] + b'\x00\x00')[0]
+
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
 
     @property
     def globalCounter(self) -> int:
@@ -159,13 +163,6 @@ class MessageID:
         """
         return self.__globalCounter == 0 and self.__replicaID == 0
 
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw bytes used to create this object.
-        """
-        return self.__rawData
-
     @property
     def replicaID(self) -> int:
         """
@@ -179,11 +176,11 @@ class ServerID:
     """
     Class representing a PtypServerId.
     """
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         """
         :param data: The data to use to create the ServerID.
 
-        :raises TypeError: if the data is not a ServerID.
+        :raises TypeError: The data is not a ServerID.
         """
         # According to the docs, the first byte being a 1 means it follows this
         # structure. A value of 0 means it does not.
@@ -192,7 +189,13 @@ class ServerID:
         self.__rawData = data
         self.__folderID = FolderID(data[1:9])
         self.__messageID = MessageID(data[9:17])
-        self.__instance = constants.STUI32.unpack(data[17:21])
+        self.__instance = constants.st.ST_LE_UI32.unpack(data[17:21])[0]
+
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
 
     @property
     def folderID(self) -> FolderID:
@@ -217,10 +220,3 @@ class ServerID:
         properties will be 0.
         """
         return self.__messageID
-
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw data used to create this object.
-        """
-        return self.__rawData
