@@ -14,7 +14,7 @@ class ReportTag:
     A Report Tag structure, as defined in [MS-OXOMSG].
     """
 
-    def __init__(self, data : bytes):
+    def __init__(self, data: bytes):
         self.__rawData = data
         reader = BytesReader(data)
 
@@ -57,17 +57,27 @@ class ReportTag:
         else:
             self.__ansiText = None
 
+    def __bytes__(self) -> bytes:
+        return self.toBytes()
+
+    def toBytes(self) -> bytes:
+        return self.__rawData
+
     @property
     def ansiText(self) -> Optional[bytes]:
         """
-        The subject of the original message. Set to None if not present.
+        The subject of the original message.
+
+        Set to None if not present.
         """
         return self.__ansiText
 
     @property
     def cookie(self) -> bytes:
         """
-        String used for validation. Set to b'PCDFEB09\x00'.
+        String used for validation.
+
+        Set to ``b'PCDFEB09\x00'``.
         """
         return self.__cookie
 
@@ -92,13 +102,6 @@ class ReportTag:
         """
         return self.__messageSearchKey
 
-    @property
-    def rawData(self) -> bytes:
-        """
-        The raw bytes used to create this object.
-        """
-        return self.__rawData
-
     @property
     def searchFolderEntryID(self) -> Optional[EntryID]:
         """
@@ -116,7 +119,9 @@ class ReportTag:
     @property
     def version(self) -> int:
         """
-        The version used. If SearchFolderEntryID is present, this MUST be
-        0x00020001, otherwise it MUST be 0x00010001.
+        The version used.
+
+        If SearchFolderEntryID is present, this MUST be ``0x00020001``,
+        otherwise it MUST be ``0x00010001``.
         """
         return self.__version
